@@ -13,8 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import argparse
+import itertools
 import logging
 import netrc
+import operator
 import os
 import subprocess
 import sys
@@ -97,7 +99,28 @@ def get_github_user_repo_from_url(url):
     return user, repo[:-4]
 
 
-def git_pull_request(remote_branch=None, title=None):
+def split_and_remove_empty_lines(s):
+    return filter(operator.truth, s.split("\n"))
+
+
+def parse_pr_message(message):
+    message_by_line = message.split("\n")
+    if len(message) == 0:
+        return None, None
+    title = message_by_line[0]
+    body = "\n".join(itertools.dropwhile(
+        operator.not_, message_by_line[1:]))
+    return title, body
+
+
+def get_title_from_git_log(log, branch):
+    summary_entries = list(split_and_remove_empty_lines(log))
+    if len(summary_entries) == 1:
+        return summary_entries[0]
+    return "Pull request for " + branch
+
+
+def git_pull_request(remote=None, remote_branch=None, title=None):
     branch = git_get_branch_name()
     if not branch:
         LOG.critical("Unable to find current branch")
@@ -105,7 +128,7 @@ def git_pull_request(remote_branch=None, title=None):
 
     LOG.debug("Local branch name is `%s'", branch)
 
-    remote = git_get_remote_for_branch(branch)
+    remote = remote or git_get_remote_for_branch(branch)
     if not remote:
         LOG.debug(
             "Unable to find remote for local branch `%s', using master",
@@ -117,7 +140,7 @@ def git_pull_request(remote_branch=None, title=None):
                 branch)
             return 20
 
-    LOG.debug("Remote name for branch `%s' is `%s'", branch, remote)
+    LOG.debug("Remote for branch `%s' is `%s'", branch, remote)
 
     remote_url = git_remote_url(remote)
     if not remote_url:
@@ -167,7 +190,10 @@ def git_pull_request(remote_branch=None, title=None):
     pulls = list(repo_to_fork.get_pulls(head=user + ":" + branch))
     if pulls:
         for pull in pulls:
-            LOG.info("Pull-request already exists at: " + pull.html_url)
+            LOG.info("Pull-request already exists at: %s", pull.html_url)
+            if title:
+                pull.edit(title=title)
+                LOG.info("Update pull-request title to: %s", title)
     else:
         remote_branch = (remote_branch or
                          git_get_remote_branch_for_branch(branch))
@@ -185,12 +211,7 @@ def git_pull_request(remote_branch=None, title=None):
              remote + "/" + remote_branch + ".." + branch],
             output=True)
 
-        if title is None:
-            summary_entries = summary_log.split("\n")
-            if len(summary_entries) == 1:
-                title = summary_entries[0]
-            else:
-                title = "Merge request for " + branch
+        title = title or get_title_from_git_log(summary_log, branch)
 
         fd, bodyfilename = tempfile.mkstemp()
         with open(bodyfilename, "w") as body:
@@ -201,12 +222,9 @@ def git_pull_request(remote_branch=None, title=None):
             content = body.read().strip()
         os.unlink(bodyfilename)
 
-        content_by_line = content.split("\n")
-        if len(content_by_line) > 0:
-            title = content_by_line[0]
-            body = "\n".join(content_by_line[1:])
-        else:
-            LOG.warning("Pull-request message is empty, aborting")
+        title, body = parse_pr_message(content)
+        if title is None:
+            LOG.critical("Pull-request message is empty, aborting")
             return 40
 
         pull = repo_to_fork.create_pull(base=remote_branch,
@@ -217,18 +235,15 @@ def git_pull_request(remote_branch=None, title=None):
 
 
 def main():
-    daiquiri.setup(
-        outputs=(
-            daiquiri.output.Stream(
-                sys.stdout,
-                formatter=logging.Formatter(
-                    fmt="%(message)s")),),
-        level=logging.INFO,
-    )
-
     parser = argparse.ArgumentParser(
         description='Send GitHub pull-request.'
     )
+    parser.add_argument("--debug",
+                        action='store_true',
+                        help="Enabled debugging.")
+    parser.add_argument("--remote",
+                        help="Remote to send a pull-request to. "
+                        "Default is auto-detected from .git/config.")
     parser.add_argument("--remote-branch",
                         help="Remote branch to send a pull-request to. "
                         "Default is auto-detected from .git/config.")
@@ -237,8 +252,18 @@ def main():
 
     args = parser.parse_args()
 
-    git_pull_request(remote_branch=args.remote_branch,
-                     title=args.title)
+    daiquiri.setup(
+        outputs=(
+            daiquiri.output.Stream(
+                sys.stdout,
+                formatter=logging.Formatter(
+                    fmt="%(message)s")),),
+        level=logging.DEBUG if args.debug else logging.INFO,
+    )
+
+    return git_pull_request(remote=args.remote,
+                            remote_branch=args.remote_branch,
+                            title=args.title)
 
 
 if __name__ == '__main__':
